module.exports = (function() {
  "use strict";

  var yauzl = require("yauzl");
  var fs = require("fs");
  var path = require("path");
  var mkdirp = require("mkdirp");

  var unzip = function(archive, destination, cb) {
    try {
      yauzl.open(archive, {lazyEntries: true}, function(err, zipfile) {
        if (err) throw err;
        zipfile.readEntry();
        zipfile.on("entry", function(entry) {
          var fullPath = path.join(destination, entry.fileName);
          // directory file names end with '/'
          if (/\/$/.test(entry.fileName)) {
            mkdirp(fullPath, function(err) {
              if (err) throw err;
              zipfile.readEntry();
            });
          } else {
            // file entry
            zipfile.openReadStream(entry, function(err, readStream) {
              if (err) throw err;
              // ensure parent directory exists
              mkdirp(path.dirname(fullPath), function(err) {
                if (err) throw err;
                readStream.pipe(fs.createWriteStream(fullPath));
                readStream.on("end", function() {
                  zipfile.readEntry();
                });
              });
            });
          }
        });
        zipfile.on("end", function() {
          cb();
        });
      });
    } catch (err) {
      cb(err);
    }
  };

  return unzip;
}());