module.exports = (function() {
  "use strict";

  var fs = require("fs");
  var util = require("util");
  var _ = require("lodash");
  var assert = require("assert");
  var outputType = {
    DEBUG: 1,     // STDOUT - diagnostic only, not fed to TDX
    ERROR: 2,     // STDERR - fed back to TDX
    RESULT: 3,    // Result update to the TDX
    PROGRESS: 4,  // Progress updates to TDX
  };

  var _writeOutput = function(fd, msg) {
    msg = typeof msg !== "undefined" ? msg : "";
    var buf = new Buffer(msg.toString());
    fs.write(fd, buf, 0, buf.length);
  };

  var writeResult = function(obj) {
    if (typeof obj !== "object") {
      return writeError("output.result - expected type 'object', got type '%s'", typeof obj);
    } else {
      return _writeOutput(outputType.RESULT, JSON.stringify(obj));
    }
  };

  var writeDebug = function() {
    var msg = util.format.apply(util, arguments);
    return _writeOutput(outputType.DEBUG, msg + "\n");
  };

  var writeError = function() {
    var msg = util.format.apply(util, arguments);
    return _writeOutput(outputType.ERROR, msg + "\n");
  };

  var writeProgress = function(progress) {
    assert(_.isNumber(progress));
    return _writeOutput(outputType.PROGRESS, progress.toString());
  };

  return {
    debug: writeDebug,
    progress: writeProgress,
    error: writeError,
    result: writeResult
  }
}());